<?php

namespace Drupal\wordpress_migrate_sql\Plugin\migrate\source;

use Drupal\migrate\Row;

/**
 * Extract content from Wordpress site.
 *
 * @MigrateSource(
 *   id = "wordpress_migrate_sql_posts"
 * )
 */
class Posts extends SqlBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    $query = $this->select($this->getTableName('posts'), 'p');

    $query
      ->fields('p', [
        'id',
        'post_date',
        'post_title',
        'post_content',
        'post_excerpt',
        'post_modified',
        'post_name',
        'post_status',
      ]);

    $query->condition('p.post_type', $this->getPostType());
    if (isset($this->configuration['filter']['status']) && is_array($this->configuration['filter']['status'])) {
      $query->condition('p.post_status', $this->configuration['filter']['status'], 'IN');
    }

    if (isset($this->configuration['filter']['terms']) && is_array($this->configuration['filter']['terms'])) {
      $query->join('term_relationships', 'tr', 'tr.object_id = p.id');
      $query->condition('tr.term_taxonomy_id', $this->configuration['filter']['terms'], 'IN');
    }

    $query->groupBy('p.id');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    return [
      'id'            => $this->t('Post ID'),
      'post_title'    => $this->t('Title'),
      'post_excerpt'  => $this->t('Excerpt'),
      'post_content'  => $this->t('Content'),
      'post_status'  => $this->t('Post Status'),
      'post_date'     => $this->t('Created Date'),
      'post_modified' => $this->t('Modified Date'),
      'path_name'     => $this->t('URL Alias'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return [
      'id' => [
        'type'  => 'integer',
        'alias' => 'p',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    $row->setSourceProperty('post_tags', $this->getTaxonomyTags($row, 'post_tag'));
    $row->setSourceProperty('category_tags', $this->getTaxonomyTags($row, 'category'));
    $row->setSourceProperty('thumbnail_id', $this->getThumbnail($row));
  }

  /**
   * Get the related tags with this post.
   *
   * @param \Drupal\migrate\Row $row
   *   Row.
   * @param string $taxonomy
   *   Machine name of the taxonomy.
   *
   * @return array
   *   List of related tags.
   */
  protected function getTaxonomyTags(Row $row, string $taxonomy) {
    $query = $this->select($this->getTableName('term_relationships'), 'tr')
      ->fields('tr', ['term_taxonomy_id']);

    $query->join($this->getTableName('term_taxonomy'), 'tt', 'tr.term_taxonomy_id = tt.term_taxonomy_id and tt.taxonomy = :tag_taxonomy', [
      ':tag_taxonomy' => $taxonomy,
    ]);
    $query->condition('object_id', $row->get('id'));

    return array_values($query->execute()->fetchAllKeyed(0, 0));
  }

  /**
   * Get the related thumbnail with this post.
   *
   * @param \Drupal\migrate\Row $row
   *   Row.
   *
   * @return array
   *   Thumbnail ID.
   */
  protected function getThumbnail(Row $row) {
    return $this->getPostMetaValue($row->get('id'), '_thumbnail_id');
  }

  /**
  * Get the related Metavalue with this post.
  *
  * @param \Drupal\migrate\Row $row
  *   Row.
  *
  * @return array
  *   Metavalue.
  */
  protected function getPostMetaValue(string $post_id, string $meta_key) {
    $query = $this->select($this->getTableName('posts'), 'p');
    $query->addField('pm', 'meta_value');
    $query->join(
      $this->getTableName('postmeta'),
        'pm',
        'p.id = pm.post_id and pm.meta_key = :attached_file',
        [':meta_key' => $meta_key],
      );
    $query->condition('p.id', $post_id);
    return array_values($query->execute()->fetchAllKeyed(0, 0));
  }

  /**
   * Get Wordpress post type from the migration template.
   */
  protected function getPostType() {
    return !empty($this->configuration['post_type']) ? $this->configuration['post_type'] : 'post';
  }

}
